@@ -22,6 +22,26 @@ var selections = [];
 var reload = false;
 
 $(function() {
+    $('#tasktable').bootstrapTable({
+        formatNoMatches: function () {
+            return '';
+        },
+        striped: true
+    });
+    if ($('#tasktable').length) {
+        setInterval(function () {
+            $.ajax({
+                method: "get",
+                url: getPath() + "/ajax/emailstat",
+                async: true,
+                timeout: 900,
+                success: function (data) {
+                    $('#tasktable').bootstrapTable("load", data);
+                }
+            });
+        }, 1000);
+    }
+
     $("#books-table").on("check.bs.table check-all.bs.table uncheck.bs.table uncheck-all.bs.table",
         function (e, rowsAfter, rowsBefore) {
             var rows = rowsAfter;
@@ -46,9 +66,14 @@ $(function() {
             if (selections.length < 1) {
                 $("#delete_selection").addClass("disabled");
                 $("#delete_selection").attr("aria-disabled", true);
+                $("#table_xchange").addClass("disabled");
+                $("#table_xchange").attr("aria-disabled", true);
             } else {
                 $("#delete_selection").removeClass("disabled");
                 $("#delete_selection").attr("aria-disabled", false);
+                $("#table_xchange").removeClass("disabled");
+                $("#table_xchange").attr("aria-disabled", false);
+
             }
         });
     $("#delete_selection").click(function() {
@@ -69,7 +94,12 @@ $(function() {
         });
     });
 
-    $("#merge_books").click(function() {
+    $("#merge_books").click(function(event) {
+        if ($(this).hasClass("disabled")) {
+            event.stopPropagation()
+        } else {
+            $('#mergeModal').modal("show");
+        }
         $.ajax({
             method:"post",
             contentType: "application/json; charset=utf-8",
@@ -86,6 +116,20 @@ $(function() {
         });
     });
 
+    $("#table_xchange").click(function() {
+        $.ajax({
+            method:"post",
+            contentType: "application/json; charset=utf-8",
+            dataType: "json",
+            url: window.location.pathname + "/../../ajax/xchange",
+            data: JSON.stringify({"xchange":selections}),
+            success: function success() {
+                $("#books-table").bootstrapTable("refresh");
+                $("#books-table").bootstrapTable("uncheckAll");
+            }
+        });
+    });
+
     var column = [];
     $("#books-table > thead > tr > th").each(function() {
         var element = {};
@@ -580,13 +624,21 @@ function singleUserFormatter(value, row) {
     return '<a class="btn btn-default" onclick="storeLocation()" href="' + window.location.pathname + '/../../admin/user/' + row.id + '">' + this.buttontext + '</a>'
 }
 
-function checkboxFormatter(value, row, index){
+function checkboxFormatter(value, row){
     if(value & this.column)
         return '<input type="checkbox" class="chk" data-pk="' + row.id + '" data-name="' + this.field + '" checked onchange="checkboxChange(this, ' + row.id + ', \'' + this.name + '\', ' + this.column + ')">';
     else
         return '<input type="checkbox" class="chk" data-pk="' + row.id + '" data-name="' + this.field + '" onchange="checkboxChange(this, ' + row.id + ', \'' + this.name + '\', ' + this.column + ')">';
 }
 
+function singlecheckboxFormatter(value, row){
+    if(value)
+        return '<input type="checkbox" class="chk" data-pk="' + row.id + '" data-name="' + this.field + '" checked onchange="checkboxChange(this, ' + row.id + ', \'' + this.name + '\', 0)">';
+    else
+        return '<input type="checkbox" class="chk" data-pk="' + row.id + '" data-name="' + this.field + '" onchange="checkboxChange(this, ' + row.id + ', \'' + this.name + '\', 0)">';
+}
+
+
 /* Do some hiding disabling after user list is loaded */
 function loadSuccess() {
     var guest = $(".editable[data-name='name'][data-value='Guest']");
@@ -635,33 +687,34 @@ function move_header_elements() {
         }
     });
     $(".multi_selector").selectpicker();
-
-    if (! $._data($(".multi_head").get(0), "events") ) {
-        // Functions have to be here, otherwise the callbacks are not fired if visible columns are changed
-        $(".multi_head").on("click", function () {
-            var val = $(this).data("set");
-            var field = $(this).data("name");
-            var result = $('#user-table').bootstrapTable('getSelections').map(a => a.id);
-            var values = $("#" + field).val();
-            confirmDialog(
-                "restrictions",
-                "GeneralChangeModal",
-                0,
-                function () {
-                    $.ajax({
-                        method: "post",
-                        url: window.location.pathname + "/../../ajax/editlistusers/" + field,
-                        data: {"pk": result, "value": values, "action": val},
-                        success: function (data) {
-                            handleListServerResponse(data);
-                        },
-                        error: function (data) {
-                            handleListServerResponse([{type: "danger", message: data.responseText}])
-                        },
-                    });
-                }
-            );
-        });
+    if ($(".multi_head").length) {
+        if (!$._data($(".multi_head").get(0), "events")) {
+            // Functions have to be here, otherwise the callbacks are not fired if visible columns are changed
+            $(".multi_head").on("click", function () {
+                var val = $(this).data("set");
+                var field = $(this).data("name");
+                var result = $('#user-table').bootstrapTable('getSelections').map(a => a.id);
+                var values = $("#" + field).val();
+                confirmDialog(
+                    "restrictions",
+                    "GeneralChangeModal",
+                    0,
+                    function () {
+                        $.ajax({
+                            method: "post",
+                            url: window.location.pathname + "/../../ajax/editlistusers/" + field,
+                            data: {"pk": result, "value": values, "action": val},
+                            success: function (data) {
+                                handleListServerResponse(data);
+                            },
+                            error: function (data) {
+                                handleListServerResponse([{type: "danger", message: data.responseText}])
+                            },
+                        });
+                    }
+                );
+            });
+        }
     }
 
     $("#user_delete_selection").click(function () {
@@ -673,38 +726,41 @@ function move_header_elements() {
     $("#select_default_language").on("change", function () {
         selectHeader(this, "default_language");
     });
-
-    if (! $._data($(".check_head").get(0), "events") ) {
-        $(".check_head").on("change", function () {
-            var val = $(this).data("set");
-            var name = $(this).data("name");
-            var data = $(this).data("val");
-            checkboxHeader(val, name, data);
-        });
+    if ($(".check_head").length) {
+        if (!$._data($(".check_head").get(0), "events")) {
+            $(".check_head").on("change", function () {
+                var val = $(this).data("set");
+                var name = $(this).data("name");
+                var data = $(this).data("val");
+                checkboxHeader(val, name, data);
+            });
+        }
     }
-    if (! $._data($(".button_head").get(0), "events") ) {
-        $(".button_head").on("click", function () {
-            var result = $('#user-table').bootstrapTable('getSelections').map(a => a.id);
-            confirmDialog(
-                "btndeluser",
-                "GeneralDeleteModal",
-                0,
-                function () {
-                    $.ajax({
-                        method: "post",
-                        url: window.location.pathname + "/../../ajax/deleteuser",
-                        data: {"userid": result},
-                        success: function (data) {
-                            selections = selections.filter((el) => !result.includes(el));
-                            handleListServerResponse(data);
-                        },
-                        error: function (data) {
-                            handleListServerResponse([{type: "danger", message: data.responseText}])
-                        },
-                    });
-                }
-            );
-        });
+    if ($(".button_head").length) {
+        if (!$._data($(".button_head").get(0), "events")) {
+            $(".button_head").on("click", function () {
+                var result = $('#user-table').bootstrapTable('getSelections').map(a => a.id);
+                confirmDialog(
+                    "btndeluser",
+                    "GeneralDeleteModal",
+                    0,
+                    function () {
+                        $.ajax({
+                            method: "post",
+                            url: window.location.pathname + "/../../ajax/deleteuser",
+                            data: {"userid": result},
+                            success: function (data) {
+                                selections = selections.filter((el) => !result.includes(el));
+                                handleListServerResponse(data);
+                            },
+                            error: function (data) {
+                                handleListServerResponse([{type: "danger", message: data.responseText}])
+                            },
+                        });
+                    }
+                );
+            });
+        }
     }
 }
 
