@@ -27,6 +27,8 @@ from flask import session as flask_session
 from binascii import hexlify
 
 from flask_login import AnonymousUserMixin, current_user
+from flask_login import user_logged_in
+from contextlib import contextmanager
 
 try:
     from flask_dance.consumer.backend.sqla import OAuthConsumerMixin
@@ -61,6 +63,47 @@ Base = declarative_base()
 searched_ids = {}
 
 
+def signal_store_user_session(object, user):
+    store_user_session()
+
+def store_user_session():
+    if flask_session.get('_user_id', ""):
+        try:
+            if not check_user_session(flask_session.get('_user_id', ""), flask_session.get('_id', "")):
+                user_session = User_Sessions(flask_session.get('_user_id', ""), flask_session.get('_id', ""))
+                session.add(user_session)
+                session.commit()
+                log.debug("Login and store session : " + flask_session.get('_id', ""))
+            else:
+                log.debug("Found stored session: " + flask_session.get('_id', ""))
+        except (exc.OperationalError, exc.InvalidRequestError) as e:
+            session.rollback()
+            log.exception(e)
+    else:
+        log.error("No user id in session")
+
+def delete_user_session(user_id, session_key):
+    try:
+        log.debug("Deleted session_key: " + session_key)
+        session.query(User_Sessions).filter(User_Sessions.user_id==user_id,
+                                            User_Sessions.session_key==session_key).delete()
+        session.commit()
+    except (exc.OperationalError, exc.InvalidRequestError):
+        session.rollback()
+        log.exception(e)
+
+
+def check_user_session(user_id, session_key):
+    try:
+        return bool(session.query(User_Sessions).filter(User_Sessions.user_id==user_id,
+                                                       User_Sessions.session_key==session_key).one_or_none())
+    except (exc.OperationalError, exc.InvalidRequestError):
+        session.rollback()
+        log.exception(e)
+
+
+user_logged_in.connect(signal_store_user_session)
+
 def store_ids(result):
     ids = list()
     for element in result:
@@ -72,7 +115,7 @@ class UserBase:
 
     @property
     def is_authenticated(self):
-        return True
+        return self.is_active
 
     def _has_role(self, role_flag):
         return constants.has_flag(self.role, role_flag)
@@ -188,7 +231,7 @@ class User(UserBase, Base):
     allowed_column_value = Column(String, default="")
     remote_auth_token = relationship('RemoteAuthToken', backref='user', lazy='dynamic')
     view_settings = Column(JSON, default={})
-
+    kobo_only_shelves_sync = Column(Integer, default=0)
 
 
 if oauth_support:
@@ -229,6 +272,7 @@ class Anonymous(AnonymousUserMixin, UserBase):
         self.denied_column_value = data.denied_column_value
         self.allowed_column_value = data.allowed_column_value
         self.view_settings = data.view_settings
+        self.kobo_only_shelves_sync = data.kobo_only_shelves_sync
 
 
     def role_admin(self):
@@ -260,6 +304,17 @@ class Anonymous(AnonymousUserMixin, UserBase):
             flask_session['view'][page][prop] = value
         return None
 
+class User_Sessions(Base):
+    __tablename__ = 'user_session'
+
+    id = Column(Integer, primary_key=True)
+    user_id = Column(Integer, ForeignKey('user.id'))
+    session_key = Column(String, default="")
+
+    def __init__(self, user_id, session_key):
+        self.user_id = user_id
+        self.session_key = session_key
+
 
 # Baseclass representing Shelfs in calibre-web in app.db
 class Shelf(Base):
@@ -270,6 +325,7 @@ class Shelf(Base):
     name = Column(String)
     is_public = Column(Integer, default=0)
     user_id = Column(Integer, ForeignKey('user.id'))
+    kobo_sync = Column(Boolean, default=False)
     books = relationship("BookShelf", backref="ub_shelf", cascade="all, delete-orphan", lazy="dynamic")
     created = Column(DateTime, default=datetime.datetime.utcnow)
     last_modified = Column(DateTime, default=datetime.datetime.utcnow, onupdate=datetime.datetime.utcnow)
@@ -483,11 +539,12 @@ def migrate_registration_table(engine, session):
 
 
 # Remove login capability of user Guest
-def migrate_guest_password(engine, session):
+def migrate_guest_password(engine):
     try:
         with engine.connect() as conn:
+            trans = conn.begin()
             conn.execute(text("UPDATE user SET password='' where name = 'Guest' and password !=''"))
-        session.commit()
+            trans.commit()
     except exc.OperationalError:
         print('Settings database is not writeable. Exiting...')
         sys.exit(2)
@@ -502,6 +559,7 @@ def migrate_shelfs(engine, session):
             conn.execute("ALTER TABLE shelf ADD column 'created' DATETIME")
             conn.execute("ALTER TABLE shelf ADD column 'last_modified' DATETIME")
             conn.execute("ALTER TABLE book_shelf_link ADD column 'date_added' DATETIME")
+            conn.execute("ALTER TABLE shelf ADD column 'kobo_sync' BOOLEAN DEFAULT false")
         for shelf in session.query(Shelf).all():
             shelf.uuid = str(uuid.uuid4())
             shelf.created = datetime.datetime.now()
@@ -509,6 +567,15 @@ def migrate_shelfs(engine, session):
         for book_shelf in session.query(BookShelf).all():
             book_shelf.date_added = datetime.datetime.now()
         session.commit()
+
+    try:
+        session.query(exists().where(Shelf.kobo_sync)).scalar()
+    except exc.OperationalError:
+        with engine.connect() as conn:
+
+            conn.execute("ALTER TABLE shelf ADD column 'kobo_sync' BOOLEAN DEFAULT false")
+        session.commit()
+
     try:
         session.query(exists().where(BookShelf.order)).scalar()
     except exc.OperationalError:  # Database is not compatible, some columns are missing
@@ -592,6 +659,13 @@ def migrate_Database(session):
         with engine.connect() as conn:
             conn.execute("ALTER TABLE user ADD column `view_settings` VARCHAR(10) DEFAULT '{}'")
         session.commit()
+    try:
+        session.query(exists().where(User.kobo_only_shelves_sync)).scalar()
+    except exc.OperationalError:
+        with engine.connect() as conn:
+            conn.execute("ALTER TABLE user ADD column `kobo_only_shelves_sync` SMALLINT DEFAULT 0")
+        session.commit()
+
     try:
         # check if name is in User table instead of nickname
         session.query(exists().where(User.name)).scalar()
@@ -611,15 +685,16 @@ def migrate_Database(session):
                      "allowed_tags VARCHAR,"
                      "denied_column_value VARCHAR,"
                      "allowed_column_value VARCHAR,"
-                     "view_settings JSON,"         
+                     "view_settings JSON,"
+                     "kobo_only_shelves_sync SMALLINT,"                              
                      "UNIQUE (name),"
                      "UNIQUE (email))"))
             conn.execute(text("INSERT INTO user_id(id, name, email, role, password, kindle_mail,locale,"
                      "sidebar_view, default_language, denied_tags, allowed_tags, denied_column_value, "
-                     "allowed_column_value, view_settings)"
+                     "allowed_column_value, view_settings, kobo_only_shelves_sync)"
                      "SELECT id, nickname, email, role, password, kindle_mail, locale,"
                      "sidebar_view, default_language, denied_tags, allowed_tags, denied_column_value, "
-                     "allowed_column_value, view_settings FROM user"))
+                     "allowed_column_value, view_settings, kobo_only_shelves_sync FROM user"))
             # delete old user table and rename new user_id table to user:
             conn.execute(text("DROP TABLE user"))
             conn.execute(text("ALTER TABLE user_id RENAME TO user"))
@@ -628,7 +703,7 @@ def migrate_Database(session):
        is None:
         create_anonymous_user(session)
 
-    migrate_guest_password(engine, session)
+    migrate_guest_password(engine)
 
 
 def clean_database(session):
