@@ -84,7 +84,7 @@ except ImportError:
 
 @app.after_request
 def add_security_headers(resp):
-    # resp.headers['Content-Security-Policy']= "script-src 'self' https://www.googleapis.com https://api.douban.com https://comicvine.gamespot.com;"
+    # resp.headers['Content-Security-Policy']= "script-src 'self'" https://www.googleapis.com https://api.douban.com https://comicvine.gamespot.com;"
     resp.headers['X-Content-Type-Options'] = 'nosniff'
     resp.headers['X-Frame-Options'] = 'SAMEORIGIN'
     resp.headers['X-XSS-Protection'] = '1; mode=block'
@@ -360,9 +360,9 @@ def get_sort_function(sort, data):
     if sort == 'old':
         order = [db.Books.timestamp]
     if sort == 'authaz':
-        order = [db.Books.author_sort.asc()]
+        order = [db.Books.author_sort.asc(), db.Series.name, db.Books.series_index]
     if sort == 'authza':
-        order = [db.Books.author_sort.desc()]
+        order = [db.Books.author_sort.desc(), db.Series.name.desc(), db.Books.series_index.desc()]
     if sort == 'seriesasc':
         order = [db.Books.series_index.asc()]
     if sort == 'seriesdesc':
@@ -410,7 +410,10 @@ def render_books_list(data, sort, book_id, page):
         return render_adv_search_results(term, offset, order, config.config_books_per_page)
     else:
         website = data or "newest"
-        entries, random, pagination = calibre_db.fill_indexpage(page, 0, db.Books, True, order)
+        entries, random, pagination = calibre_db.fill_indexpage(page, 0, db.Books, True, order,
+                                                                db.books_series_link,
+                                                                db.Books.id == db.books_series_link.c.book,
+                                                                db.Series)
         return render_title_template('index.html', random=random, entries=entries, pagination=pagination,
                                      title=_(u"Books"), page=website)
 
@@ -420,7 +423,11 @@ def render_rated_books(page, book_id, order):
         entries, random, pagination = calibre_db.fill_indexpage(page, 0,
                                                                 db.Books,
                                                                 db.Books.ratings.any(db.Ratings.rating > 9),
-                                                                order)
+                                                                order,
+                                                                db.books_series_link,
+                                                                db.Books.id == db.books_series_link.c.book,
+                                                                db.Series)
+
         return render_title_template('index.html', random=random, entries=entries, pagination=pagination,
                                      id=book_id, title=_(u"Top Rated Books"), page="rated")
     else:
@@ -509,8 +516,10 @@ def render_author_books(page, author_id, order):
         flash(_(u"Oops! Selected book title is unavailable. File does not exist or is not accessible"),
               category="error")
         return redirect(url_for("web.index"))
-
-    author = calibre_db.session.query(db.Authors).get(author_id)
+    if constants.sqlalchemy_version2:
+        author = calibre_db.session.get(db.Authors, author_id)
+    else:
+        author = calibre_db.session.query(db.Authors).get(author_id)
     author_name = author.name.replace('|', ',')
 
     author_info = None
@@ -624,6 +633,9 @@ def render_read_books(page, are_read, as_xml=False, order=None):
                                                                 db.Books,
                                                                 db_filter,
                                                                 order,
+                                                                db.books_series_link,
+                                                                db.Books.id == db.books_series_link.c.book,
+                                                                db.Series,
                                                                 ub.ReadBook, db.Books.id == ub.ReadBook.book_id)
     else:
         try:
@@ -635,6 +647,9 @@ def render_read_books(page, are_read, as_xml=False, order=None):
                                                                     db.Books,
                                                                     db_filter,
                                                                     order,
+                                                                    db.books_series_link,
+                                                                    db.Books.id == db.books_series_link.c.book,
+                                                                    db.Series,
                                                                     db.cc_classes[config.config_read_column])
         except (KeyError, AttributeError):
             log.error("Custom Column No.%d is not existing in calibre database", config.config_read_column)
@@ -713,7 +728,8 @@ def render_prepare_search_form(cc):
 
 
 def render_search_results(term, offset=None, order=None, limit=None):
-    entries, result_count, pagination = calibre_db.get_search_results(term, offset, order, limit)
+    join = db.books_series_link, db.Books.id == db.books_series_link.c.book, db.Series
+    entries, result_count, pagination = calibre_db.get_search_results(term, offset, order, limit, *join)
     return render_title_template('search.html',
                                  searchterm=term,
                                  pagination=pagination,
@@ -775,11 +791,13 @@ def list_books():
         order = [db.Publishers.name.asc()] if order == "asc" else [db.Publishers.name.desc()]
         join = db.books_publishers_link,db.Books.id == db.books_publishers_link.c.book, db.Publishers
     elif sort == "authors":
-        order = [db.Authors.name.asc()] if order == "asc" else [db.Authors.name.desc()]
-        join = db.books_authors_link,db.Books.id == db.books_authors_link.c.book, db.Authors
+        order = [db.Authors.name.asc(), db.Series.name, db.Books.series_index] if order == "asc" \
+            else [db.Authors.name.desc(), db.Series.name.desc(), db.Books.series_index.desc()]
+        join = db.books_authors_link, db.Books.id == db.books_authors_link.c.book, db.Authors, \
+               db.books_series_link, db.Books.id == db.books_series_link.c.book, db.Series
     elif sort == "languages":
         order = [db.Languages.lang_code.asc()] if order == "asc" else [db.Languages.lang_code.desc()]
-        join = db.books_languages_link,db.Books.id == db.books_languages_link.c.book, db.Languages
+        join = db.books_languages_link, db.Books.id == db.books_languages_link.c.book, db.Languages
     elif order and sort in ["sort", "title", "authors_sort", "series_index"]:
         order = [text(sort + " " + order)]
     elif not state:
@@ -793,7 +811,7 @@ def list_books():
             filtered_count = len(books)
         else:
             books = calibre_db.session.query(db.Books).filter(calibre_db.common_filters()).all()
-        entries = calibre_db.get_checkbox_sorted(books, state, off, limit,order)
+        entries = calibre_db.get_checkbox_sorted(books, state, off, limit, order)
     elif search:
         entries, filtered_count, __ = calibre_db.get_search_results(search, off, order, limit, *join)
     else:
@@ -1211,7 +1229,7 @@ def extend_search_term(searchterm,
     for key, db_element in elements.items():
         tag_names = calibre_db.session.query(db_element).filter(db_element.id.in_(tags['include_' + key])).all()
         searchterm.extend(tag.name for tag in tag_names)
-        tag_names = calibre_db.session.query(db_element).filter(db.Tags.id.in_(tags['exclude_' + key])).all()
+        tag_names = calibre_db.session.query(db_element).filter(db_element.id.in_(tags['exclude_' + key])).all()
         searchterm.extend(tag.name for tag in tag_names)
     language_names = calibre_db.session.query(db.Languages). \
         filter(db.Languages.id.in_(tags['include_language'])).all()
@@ -1242,7 +1260,9 @@ def render_adv_search_results(term, offset=None, order=None, limit=None):
 
     cc = get_cc_columns(filter_config_custom_read=True)
     calibre_db.session.connection().connection.connection.create_function("lower", 1, db.lcase)
-    q = calibre_db.session.query(db.Books).filter(calibre_db.common_filters(True))
+    q = calibre_db.session.query(db.Books).outerjoin(db.books_series_link, db.Books.id == db.books_series_link.c.book)\
+        .outerjoin(db.Series)\
+        .filter(calibre_db.common_filters(True))
 
     # parse multiselects to a complete dict
     tags = dict()
@@ -1327,7 +1347,11 @@ def render_adv_search_results(term, offset=None, order=None, limit=None):
             q = q.filter(db.Books.comments.any(func.lower(db.Comments.text).ilike("%" + description + "%")))
 
         # search custom culumns
-        q = adv_search_custom_columns(cc, term, q)
+        try:
+            q = adv_search_custom_columns(cc, term, q)
+        except AttributeError as ex:
+            log.debug_or_exception(ex)
+            flash(_("Error on search for custom columns, please restart Calibre-Web"), category="error")
 
     q = q.order_by(*order).all()
     flask_session['query'] = json.dumps(term)
@@ -1381,10 +1405,14 @@ def serve_book(book_id, book_format, anyname):
         return "File not in Database"
     log.info('Serving book: %s', data.name)
     if config.config_use_google_drive:
-        headers = Headers()
-        headers["Content-Type"] = mimetypes.types_map.get('.' + book_format, "application/octet-stream")
-        df = getFileFromEbooksFolder(book.path, data.name + "." + book_format)
-        return do_gdrive_download(df, headers, (book_format.upper() == 'TXT'))
+        try:
+            headers = Headers()
+            headers["Content-Type"] = mimetypes.types_map.get('.' + book_format, "application/octet-stream")
+            df = getFileFromEbooksFolder(book.path, data.name + "." + book_format)
+            return do_gdrive_download(df, headers, (book_format.upper() == 'TXT'))
+        except AttributeError as ex:
+            log.debug_or_exception(ex)
+            return "File Not Found"
     else:
         if book_format.upper() == 'TXT':
             try:
@@ -1392,13 +1420,13 @@ def serve_book(book_id, book_format, anyname):
                                "rb").read()
                 result = chardet.detect(rawdata)
                 return make_response(
-                    rawdata.decode(result['encoding']).encode('utf-8'))
+                    rawdata.decode(result['encoding'], 'surrogatepass').encode('utf-8', 'surrogatepass'))
             except FileNotFoundError:
+                log.error("File Not Found")
                 return "File Not Found"
         return send_from_directory(os.path.join(config.config_calibre_dir, book.path), data.name + "." + book_format)
 
 
-
 @web.route("/download/<int:book_id>/<book_format>", defaults={'anyname': 'None'})
 @web.route("/download/<int:book_id>/<book_format>/<anyname>")
 @login_required_if_no_ano
@@ -1489,9 +1517,9 @@ def register():
 
 @web.route('/login', methods=['GET', 'POST'])
 def login():
-    if not config.db_configured:
-        log.debug(u"Redirect to initial configuration")
-        return redirect(url_for('admin.basic_configuration'))
+    #if not config.db_configured:
+    #    log.debug(u"Redirect to initial configuration")
+    #    return redirect(url_for('admin.basic_configuration'))
     if current_user is not None and current_user.is_authenticated:
         return redirect(url_for('web.index'))
     if config.config_login_type == constants.LOGIN_LDAP and not services.ldap:
@@ -1564,6 +1592,7 @@ def login():
 @login_required
 def logout():
     if current_user is not None and current_user.is_authenticated:
+        ub.delete_user_session(current_user.id, flask_session.get('_id',""))
         logout_user()
         if feature_support['oauth'] and (config.config_login_type == 2 or config.config_login_type == 3):
             logout_oauth_user()
@@ -1579,26 +1608,33 @@ def change_profile(kobo_support, local_oauth_check, oauth_status, translations,
         if to_save.get("password"):
             current_user.password = generate_password_hash(to_save["password"])
     try:
-        if to_save.get("allowed_tags", current_user.allowed_tags) != current_user.allowed_tags:
-            current_user.allowed_tags = to_save["allowed_tags"].strip()
         if to_save.get("kindle_mail", current_user.kindle_mail) != current_user.kindle_mail:
             current_user.kindle_mail = valid_email(to_save["kindle_mail"])
         if to_save.get("email", current_user.email) != current_user.email:
             current_user.email = check_email(to_save["email"])
-        if to_save.get("name", current_user.name) != current_user.name:
-            # Query User name, if not existing, change
-            current_user.name = check_username(to_save["name"])
+        if current_user.role_admin():
+            if to_save.get("name", current_user.name) != current_user.name:
+                # Query User name, if not existing, change
+                current_user.name = check_username(to_save["name"])
         current_user.random_books = 1 if to_save.get("show_random") == "on" else 0
         if to_save.get("default_language"):
             current_user.default_language = to_save["default_language"]
         if to_save.get("locale"):
             current_user.locale = to_save["locale"]
+        current_user.kobo_only_shelves_sync = int(to_save.get("kobo_only_shelves_sync") == "on") or 0
+
     except Exception as ex:
         flash(str(ex), category="error")
-        return render_title_template("user_edit.html", content=current_user,
-                                     title=_(u"%(name)s's profile", name=current_user.name), page="me",
+        return render_title_template("user_edit.html",
+                                     content=current_user,
+                                     translations=translations,
+                                     profile=1,
+                                     languages=languages,
+                                     title=_(u"%(name)s's profile", name=current_user.name),
+                                     page="me",
                                      kobo_support=kobo_support,
-                                     registered_oauth=local_oauth_check, oauth_status=oauth_status)
+                                     registered_oauth=local_oauth_check,
+                                     oauth_status=oauth_status)
 
     val = 0
     for key, __ in to_save.items():
@@ -1670,28 +1706,33 @@ def read_book(book_id, book_format):
                                                              ub.Bookmark.format == book_format.upper())).first()
     if book_format.lower() == "epub":
         log.debug(u"Start epub reader for %d", book_id)
-        return render_title_template('read.html', bookid=book_id, title=_(u"Read a Book"), bookmark=bookmark)
+        return render_title_template('read.html', bookid=book_id, title=book.title, bookmark=bookmark)
     elif book_format.lower() == "pdf":
         log.debug(u"Start pdf reader for %d", book_id)
-        return render_title_template('readpdf.html', pdffile=book_id, title=_(u"Read a Book"))
+        return render_title_template('readpdf.html', pdffile=book_id, title=book.title)
     elif book_format.lower() == "txt":
         log.debug(u"Start txt reader for %d", book_id)
-        return render_title_template('readtxt.html', txtfile=book_id, title=_(u"Read a Book"))
+        return render_title_template('readtxt.html', txtfile=book_id, title=book.title)
     elif book_format.lower() == "djvu":
         log.debug(u"Start djvu reader for %d", book_id)
-        return render_title_template('readdjvu.html', djvufile=book_id, title=_(u"Read a Book"))
+        return render_title_template('readdjvu.html', djvufile=book_id, title=book.title)
     else:
         for fileExt in constants.EXTENSIONS_AUDIO:
             if book_format.lower() == fileExt:
                 entries = calibre_db.get_filtered_book(book_id)
                 log.debug(u"Start mp3 listening for %d", book_id)
                 return render_title_template('listenmp3.html', mp3file=book_id, audioformat=book_format.lower(),
-                                             title=_(u"Read a Book"), entry=entries, bookmark=bookmark)
+                                             entry=entries, bookmark=bookmark)
         for fileExt in ["cbr", "cbt", "cbz"]:
             if book_format.lower() == fileExt:
                 all_name = str(book_id)
+                title = book.title
+                if len(book.series):
+                    title = title + " - " + book.series[0].name
+                    if book.series_index:
+                        title = title + " #" + '{0:.2f}'.format(book.series_index).rstrip('0').rstrip('.')
                 log.debug(u"Start comic reader for %d", book_id)
-                return render_title_template('readcbr.html', comicfile=all_name, title=_(u"Read a Book"),
+                return render_title_template('readcbr.html', comicfile=all_name, title=title,
                                              extension=fileExt)
         log.debug(u"Oops! Selected book title is unavailable. File does not exist or is not accessible")
         flash(_(u"Oops! Selected book title is unavailable. File does not exist or is not accessible"), category="error")
